! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Compute Warshel quantities                                     |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine awarshel_dE 

   use pmf_parm, only: nwin, nbias, ndata, num_bin, bin_start &
                     , bin_width_inv, beta 

   use map_pot,  only: map_dat, awarshel, pmf


   implicit none

   !  +---------------------------------------------------------------+

   integer :: l, m, n, ni, nf, ibin

   real*8 :: dE

   real*8, intrinsic :: idint, abs, exp


!  x---------------------------------------------------------------x
!  |  Allocate memory                                              |
!  x---------------------------------------------------------------x

   call awarshel_dE_alloc

!  +---------------------------------------------------------------+
!  |  Compute factor involving <V_el0 - V_map>, <V_ii - V_map>,    |
!  |     <V_ff - V_map>                                            |
!  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!  |  < exp[ - beta * ( V_el0 - V_map ) ] >_{lambda_m, /\_n}       |
!  |  < exp[ - beta * ( V_ii  - V_map ) ] >_{lambda_m, /\_n}       |
!  |  < exp[ - beta * ( V_ff  - V_map ) ] >_{lambda_m, /\_n}       |
!  +---------------------------------------------------------------+

   do n = 1, nwin

      pmf(n)% pb(:) = 0.0d0

      pmf(n)% dE0(:) = 0.0d0
      pmf(n)% dEi(:) = 0.0d0
      pmf(n)% dEf(:) = 0.0d0

      do m = 1, ndata(n)

         ibin = idint( ( awarshel(n)% nrg_gap(m) - bin_start ) &
              *  bin_width_inv )

         if( ibin /= 0 ) then 

            pmf(n)% pb(ibin) = pmf(n)% pb(ibin) + 1.0d0

            dE = awarshel(n)% nrg_evb(m) - awarshel(n)% nrg_map(m)
            pmf(n)% dE0(ibin) = pmf(n)% dE0(ibin) + exp( - beta * dE )

            do l = 1, nbias

               ni = map_dat(n)% map_init (l)
               nf = map_dat(n)% map_final(l)

               dE = map_dat(n)% evb_mat(ni,ni,m) - awarshel(n)% nrg_map(m)
               pmf(n)% dEi(ibin) = pmf(n)% dEi(ibin) + exp( - beta * dE )

               dE = map_dat(n)% evb_mat(nf,nf,m) - awarshel(n)% nrg_map(m)
               pmf(n)% dEf(ibin) = pmf(n)% dEf(ibin) + exp( - beta * dE )

            enddo

         endif 

      enddo

   enddo

   do n = 1, nwin

      do m = 1, num_bin

         if( pmf(n)% pb(m) /= 0.0d0 ) then

            pmf(n)% dE0(m) = pmf(n)% dE0(m) / pmf(n)% pb(m) 

            pmf(n)% dEi(m) = pmf(n)% dEi(m) / pmf(n)% pb(m) 

            pmf(n)% dEf(m) = pmf(n)% dEf(m) / pmf(n)% pb(m) 

         endif 

      enddo

   enddo

!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x
!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x

   end subroutine awarshel_dE


! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Allocate storage space for awarshel_dE                         |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine awarshel_dE_alloc

   use pmf_parm, only: nwin, num_bin

   use map_pot,  only: pmf

   implicit none

   !  +---------------------------------------------------------------+

   integer :: n, alloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory for PMF calculation                          |
!  +---------------------------------------------------------------+

   allocate( pmf(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'pmf' )

   do n = 1, nwin

      allocate( pmf(n)% pb(num_bin), stat = alloc_error )
      call chk_alloc ( alloc_error, 'pb inside pmf' )

      allocate( pmf(n)% FE_el0(num_bin), stat = alloc_error )
      call chk_alloc ( alloc_error, 'FE_el0 inside pmf' )

      allocate( pmf(n)% FE_map(num_bin), stat = alloc_error )
      call chk_alloc ( alloc_error, 'FE_map inside pmf' )

      allocate( pmf(n)% FE_ids(num_bin), stat = alloc_error )
      call chk_alloc ( alloc_error, 'FE_ids inside pmf' )

      allocate( pmf(n)% FE_fds(num_bin), stat = alloc_error )
      call chk_alloc ( alloc_error, 'FE_fds inside pmf' )

      allocate( pmf(n)% dE0(num_bin), stat = alloc_error )
      call chk_alloc ( alloc_error, 'dE0 inside pmf' )

      allocate( pmf(n)% dEi(num_bin), stat = alloc_error )
      call chk_alloc ( alloc_error, 'dEi inside pmf' )

      allocate( pmf(n)% dEf(num_bin), stat = alloc_error )
      call chk_alloc ( alloc_error, 'dEf inside pmf' )

   enddo

   end subroutine awarshel_dE_alloc

