! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Read inputs for PMF                                            |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine chk_data_slices

   use pmf_parm, only: nbias, nwin, io_unit &
                     , slice_size, bias_size, data_type_size

   use map_pot,  only: map_dat 

   
   implicit none

   !  x---------------------------------------------------------------x

   integer :: k, m, n, mm, nn, ios

   character(512) :: cline 

!! Temporary arrays for checking the consistency of concatenated data slices

   integer ::     Tnbead(slice_size)                  ! # of beads in PIMD
   integer ::      Tmevb(slice_size)                  ! # of EVB states
   integer ::     Tmbias(slice_size)                  ! # of biased potentials
   integer ::  Tevb_step(slice_size)                  ! step interval for EVB output
   integer ::  Tmap_init(slice_size,bias_size)       ! Mapping potential initial state
   integer :: Tmap_final(slice_size,bias_size)       ! Mapping potential final state
   real*8  ::    Tlambda(slice_size,bias_size)       ! Mapping potential parameter
   real*8  ::     Tk_umb(slice_size)                  ! Umbrella potential force constant
   real*8  ::    Tr0_umb(slice_size)                  ! Umbrella potential anchor point
   real*8  ::        Tdt(slice_size)                  ! MD timestep size
   character(512) :: Tdyn_type(slice_size)            ! Dynamics type

   logical :: Lshit(data_type_size), Lcrap(data_type_size,bias_size) 
   logical :: Lshitting_out, Lcrapping_out 

!! Temporary arrays for checking the consistency of concatenated data slices


   Lshitting_out = .false.
   Lcrapping_out = .false. 

!  +---------------------------------------------------------------+
!  |  Read EVB input file                                          |
!  +---------------------------------------------------------------+

   Print *, 'CHECKING CONSISTENCY OF DATA SLICES' 

   do n = 1, nwin 

      open( io_unit, file = trim( adjustl(  map_dat(n)% evb_file  ) ) )

!  +---------------------------------------------------------------+
!  |  Populate temporary arrays for checking the consistency of    |
!  |  concatenated data slices for each data file                  |
!  +---------------------------------------------------------------+

      nn  = 0
      ios = 0

      do m = 1, nbias
         Tmap_init(:,m)  = 0
         Tmap_final(:,m) = 0
         Tlambda(:,m) = 0.0d0
         Tk_umb(:) = 0.0d0
         Tr0_umb(:) = 0.0d0
      enddo

      do while( ios == 0 )

         read(io_unit,'(A)', iostat=ios) cline

         select case( trim( adjustl(cline) ) )

            case( "[DYNAMICS TYPE]:" )
               nn = nn + 1
               if( nn > slice_size ) then
                  print *, 'The number of data slices in data file ' &
                         // trim( adjustl( map_dat(n)% evb_file ) ), ' is greater ' &
                         // 'than parameter slice_size.'
                  print *, '# of data slices = ', nn
                  print *, 'slice_size = ', slice_size
                  print *, 'Please increase the value of slice_size in ' &
                         // 'pmf_vars and recompile'
                  stop 6
               endif
               read(io_unit,'(A)', iostat=ios) Tdyn_type(nn)

            case( "[NBEAD]:" )
               read(io_unit,'(I8)', iostat=ios ) Tnbead(nn)

            case( "[NEV]   [NBIAS]   [NTW_EVB]   [DT]:" )
               read(io_unit, 1000, iostat=ios) Tmevb(nn), Tmbias(nn) &
                                             , Tevb_step(nn), Tdt(nn)

            case( "[NEV]   [NBIAS]   [EVB_STEP]   [DT]:" )
               read(io_unit, 1000, iostat=ios) Tmevb(nn), Tmbias(nn) &
                                             , Tevb_step(nn), Tdt(nn)

            case( "[MAPPING POTENTIAL]:   ni, nf, lambda" )
               do m = 1, nbias
                  read(io_unit,2000,iostat=ios) Tmap_init(nn,m) &
                      , Tmap_final(nn,m), Tlambda(nn,m)
               enddo

            case( "[NRG_GAP UMBRELLA]:   ni, nf, k, ezero" )
               do m = 1, nbias
                   read(io_unit,2500,iostat=ios) Tmap_init(nn,m) &
                       , Tmap_final(nn,m), Tk_umb(nn), Tr0_umb(nn)
               enddo

         end select

      enddo

      if( nn < slice_size ) then 
         Tdyn_type(nn+1:slice_size) = Tdyn_type(nn)
            Tnbead(nn+1:slice_size) = Tnbead(nn)
             Tmevb(nn+1:slice_size) = Tmevb(nn)
            Tmbias(nn+1:slice_size) = Tmbias(nn)
         Tevb_step(nn+1:slice_size) = Tevb_step(nn)
               Tdt(nn+1:slice_size) = Tdt(nn)
         do m = 1, nbias
             Tmap_init(nn+1:slice_size,m) = Tmap_init(nn,m)
            Tmap_final(nn+1:slice_size,m) = Tmap_final(nn,m)
               Tlambda(nn+1:slice_size,m) = Tlambda(nn,m)
         enddo
          Tk_umb(nn+1:slice_size) = Tk_umb(nn)
         Tr0_umb(nn+1:slice_size) = Tr0_umb(nn)
      endif 

!  +---------------------------------------------------------------+
!  |  Check that the data slices are of the same type for each     |
!  |  data file                                                    |
!  +---------------------------------------------------------------+

       Lshit(:) = .true.
       Lcrap(:,:) = .true. 

       Lshit(1) = all( Tdyn_type == Tdyn_type(1) )
       Lshit(2) = all( Tnbead == Tnbead(1) )
       Lshit(3) = all( Tmevb == Tmevb(1) )
       Lshit(4) = all( Tmbias == Tmbias(1) )
       Lshit(5) = all( Tevb_step == Tevb_step(1) )
       Lshit(6) = all( Tdt == Tdt(1) )
       Lshit(7) = all( Tk_umb == Tk_umb(1) )
       Lshit(8) = all( Tr0_umb == Tr0_umb(1) )

       do m = 1, nbias 
          Lcrap(1,m) = all( Tmap_init(:,m) == Tmap_init(1,m) )
          Lcrap(2,m) = all( Tmap_final(:,m) == Tmap_final(1,m) )
          Lcrap(3,m) = all( Tlambda(:,m) == Tlambda(1,m) )
       enddo

       do m = 1, data_type_size
          if( .not. Lshit(m) ) then
             write(6,'(/,A)') '## Data slice inconsistency in data file ' &
                    // trim( adjustl( map_dat(n)% evb_file ) )
             Lshitting_out = .true.
             select case ( m )
                case(1)
                   write(6,'(A)') "[dyn_type]"
                   write(6,'(4(A,2x))') ( trim( adjustl( Tdyn_type(k) ) ), k = 1, nn )
                case(2)
                   write(6,'(A)') "[nbead]"
                   write(6,'(4(I8,2x))') ( Tnbead(k), k = 1, nn )
                case(3) 
                   write(6,'(A)') "[mevb]"
                   write(6,'(4(I8,2x))') ( Tmevb(k), k = 1, nn ) 
                case(4)
                   write(6,'(A)') "[mbias]"
                   write(6,'(4(I8,2x))') ( Tmbias(k), k = 1, nn )
                case(5)
                   write(6,'(A)') "[evb_step]"
                   write(6,'(4(I8,2x))') ( Tevb_step(k), k = 1, nn )
                case(6)
                   write(6,'(A)') "[dt]"
                   write(6,'(4(F10.4,2x))') ( Tdt(k), k = 1, nn ) 
                case(7)
                   write(6,'(A)') "[k_umb]"
                   write(6,'(4(F10.4,2x))') ( Tk_umb(k), k = 1, nn )
                case(8)
                   write(6,'(A)') "[r0_umb]"
                   write(6,'(4(F10.4,2x))') ( Tr0_umb(k), k = 1, nn )
             end select
          endif
       enddo

       do m = 1, nbias 
          do mm = 1, data_type_size
             if( .not. Lcrap(mm,m) ) then
                write(6,'(/,A)') '## Data slice inconsistency in data file ' &
                       // trim( adjustl( map_dat(n)% evb_file ) )
                Lcrapping_out = .true.
                select case ( mm )
                   case(1)
                      write(6,'(A)') "[map_init]"
                      write(6,'(4(I8,2x))') ( Tmap_init(k,m), k = 1, nn )
                   case(2)
                      write(6,'(A)') "[map_final]"
                      write(6,'(4(I8,2x))') ( Tmap_final(k,m), k = 1, nn )
                   case(3)
                      write(6,'(A)') "[lambda]"
                      write(6,'(4(I8,2x))') ( Tlambda(k,m), k = 1, nn )
                end select
             endif
          enddo 
       enddo

       if( Lshitting_out .or. Lcrapping_out ) stop 6

       close( io_unit )

   enddo 

 1000 format( 3(2X,I8), 2X, F14.8 )
 2000 format( 2I8, F14.8 )
 2500 format( 2I8, 2F14.8 )


!  +---------------------------------------------------------------+
!  +---------------------------------------------------------------+

   end subroutine chk_data_slices




