! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Check reconstructed data against those output from Amber/EVB   |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine chk_mapdata 

   use pmf_parm, only: nwin, nbead, ndata_max, ndata &
                     , dyn_type, data_tol

   use map_pot,  only: awarshel_type, awarshel_raw 

   use pimd_dat, only: pimd_epot, vel0_epot

   implicit none


   !  +---------------------------------------------------------------+

   type( awarshel_type ), allocatable :: awarshel_tmp(:) 

   integer :: m, n, mm, nn, alloc_error, dealloc_error

   real*8 :: dyn_del, vel0_del 

   real*8, intrinsic :: abs

!  x---------------------------------------------------------------x
!  |  Allocate temporary arrays                                    |
!  x---------------------------------------------------------------x

   allocate( awarshel_tmp(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'awarshel_tmp' )

   do n = 1, nwin

      allocate( awarshel_tmp(n)% nrg_evb(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_evb in awarshel_tmp' )
      allocate( awarshel_tmp(n)% nrg_map(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_map in awarshel_tmp' )

   enddo

   select case( trim( adjustl( dyn_type ) ) )

!  +---------------------------------------------------------------+
!  |  EVB/PIMD: compute the centroid of all quantities             |
!  +---------------------------------------------------------------+

      case( "pimd_evb_map" ) 

         do n = 1, nwin

            awarshel_tmp(n)% nrg_evb(:) = 0.0d0
            awarshel_tmp(n)% nrg_map(:) = 0.0d0

            mm = 0
            do m = 1, ndata(n), nbead
               mm = mm + 1
               do nn = 0, nbead - 1
                  awarshel_tmp(n)% nrg_evb(mm) = awarshel_tmp(n)% nrg_evb(mm) &
                                               + awarshel_raw(n)% nrg_evb(m+nn)
                  awarshel_tmp(n)% nrg_map(mm) = awarshel_tmp(n)% nrg_map(mm) &
                                               + awarshel_raw(n)% nrg_map(m+nn)
               enddo 
               vel0_del = abs( awarshel_tmp(n)% nrg_evb(mm) - vel0_epot(mm,n) )
                dyn_del = abs( awarshel_tmp(n)% nrg_map(mm) - pimd_epot(mm,n) )
               if( vel0_del > data_tol ) then 
                  print *, 'ERROR:: MISMATCH BETWEEN RECONSTRUCTED DATA AND THOSE OUTPUT FROM AMBER'
                  print *, awarshel_tmp(n)% nrg_evb(mm), vel0_epot(mm,n)
                  stop 6
               endif
               if( dyn_del > data_tol ) then
                  print *, 'ERROR:: MISMATCH BETWEEN RECONSTRUCTED DATA AND THOSE OUTPUT FROM AMBER'
                  print *, awarshel_tmp(n)% nrg_map(mm), pimd_epot(mm,n)
                  stop 6
               endif
            enddo

         enddo

!  +---------------------------------------------------------------+
!  |  Conventional mapping potential EVB                           |
!  +---------------------------------------------------------------+

      case( "evb_map" )

         do n = 1, nwin 

            do m = 1, ndata(n)

               vel0_del = abs( awarshel_raw(n)% nrg_evb(m) - vel0_epot(m,n) )
!               dyn_del = abs( awarshel_raw(n)% nrg_map(m) - pimd_epot(m,n) )

               if( vel0_del > data_tol ) then
                  print *, 'ERROR:: MISMATCH BETWEEN RECONSTRUCTED DATA AND THOSE OUTPUT FROM AMBER'
                  print *, 'VEL0 :: ', awarshel_raw(n)% nrg_evb(m), vel0_epot(m,n)
                  stop 6
               endif
!              if( dyn_del > data_tol ) then
!                 print *, 'ERROR:: MISMATCH BETWEEN RECONSTRUCTED DATA AND THOSE OUTPUT FROM AMBER'
!                 print *, awarshel_raw(n)% nrg_map(m), pimd_epot(m,n)
!                 stop 6
!              endif

            enddo

         enddo 

   end select

!  x---------------------------------------------------------------x
!  |  Deallocate temporary arrays                                  |
!  x---------------------------------------------------------------x

   do n = 1, nwin
      deallocate( awarshel_tmp(n)% nrg_evb, stat = dealloc_error )
      deallocate( awarshel_tmp(n)% nrg_map, stat = dealloc_error )
   enddo

   deallocate( awarshel_tmp, stat = dealloc_error )


!  +---------------------------------------------------------------+
!  +---------------------------------------------------------------+

   end subroutine chk_mapdata


