! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Compute PMF on the EVB ground state surface                    |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine fe_pert

   use pmf_parm, only: nwin, ndata, num_bin, bin_start, bin_width &
                      , beta, beta_inv, rcrd 

   use map_pot,  only: awarshel, pmf 

   implicit none


   !  +---------------------------------------------------------------+

   integer :: j, n

   real*8 :: efmtot 

   real*8 :: efmap(num_bin,nwin)

   real*8,  intrinsic :: abs, dble, exp, log


!  +---------------------------------------------------------------+
!  |  Compute PMF as a function of energy gap reaction coordinate  |
!  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!  |  exp[ - beta * F_maptot(lambda_m) ] = sum(n) exp[ - beta      |
!  |                                     * F_map(/\_n;lambda_m) ]  |
!  |  F_maptot(lambda_m) - F_maptot(lambda_1)                      |
!  |     = sum(m'=1,m) delta lambda_m' < d V_map(r;R;lambda)       |
!  |                           /dlambda @ lambda_m' >_{lambda_m'}  |
!  |                                                               |
!  |  exp[ - beta * F_el0 ] = exp[ - beta * F_map ]                |
!  |     * < exp[ - beta * ( V_el0 - V_map) ] >_{lambda_m, /\_n}   |
!  +---------------------------------------------------------------+

   do n = 1, nwin

      efmtot = exp( -beta * awarshel(n)% FE_map )

      do j = 1, num_bin

         efmap(j,n) = efmtot * dble( pmf(n)% pb(j) ) / dble( ndata(n) )

         if( efmap(j,n) > 0.0d0 ) &
            pmf(n)% FE_map(j) = - log( efmap(j,n) ) * beta_inv

         if( efmap(j,n) * pmf(n)% dE0(j) > 0.0d0 ) &
            pmf(n)% FE_el0(j) = - log( efmap(j,n) * pmf(n)% dE0(j) ) * beta_inv

         if( efmap(j,n) * pmf(n)% dEi(j) > 0.0d0 ) &
            pmf(n)% FE_ids(j) = - log( efmap(j,n) * pmf(n)% dEi(j) ) * beta_inv

         if( efmap(j,n) * pmf(n)% dEf(j) > 0.0d0 ) &
            pmf(n)% FE_fds(j) = - log( efmap(j,n) * pmf(n)% dEf(j) ) * beta_inv

      enddo

   enddo

!  +---------------------------------------------------------------+
!  +---------------------------------------------------------------+

   end subroutine fe_pert




