! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Generate energy gap RC, mapping potential, and EVB             |#
! #|  ground-state energy                                            |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine gen_map_pot

   use pmf_parm, only: nevb, nbias, nwin, ndata, rcrd 

   use map_pot,  only: map_dat_raw, map_dat, awarshel_raw 

   implicit none


   !  +---------------------------------------------------------------+

   integer :: l, m, n, ni, nf, info

   real*8 :: evb_vec(nevb,nevb), nrg_evb(nevb) 

   real*8 :: work(26*nevb), lambda

!  x---------------------------------------------------------------x
!  |  Allocate memory                                              |
!  x---------------------------------------------------------------x

   call gen_map_pot_alloc

!  x---------------------------------------------------------------x
!  |  Loop through all the windows and data steps                  |
!  x---------------------------------------------------------------x

   do n = 1, nwin 

      do m = 1, ndata(n)

!  +---------------------------------------------------------------+
!  |  Diagonalize EVB matrix using LAPACK DSYEV                    |
!  +---------------------------------------------------------------+

         evb_vec(:,:) = map_dat_raw(n)% evb_mat(:,:,m)

         call dsyev ( 'V', 'L', nevb, evb_vec, nevb, nrg_evb, work &
                    , 26*nevb, info )

         if( info /= 0 ) then
            write(6,'(A)') 'Problem diagonalizing the EVB matrix using LAPACK DSYEV'
            write(6,'(A, I8)') 'info = ', info
            stop 6
         endif

         awarshel_raw(n)% nrg_evb(m) = nrg_evb(1)

!  +---------------------------------------------------------------+
!  |  Compute mapping potential and derivative aafo lambda         |
!  |                                                               |
!  |  V_lambda = ( 1 - lambda ) * V_ni + lambda * V_nf             |
!  |  dVmapdlam = V_nf - V_ni                                      |
!  +---------------------------------------------------------------+

         do l = 1, nbias
  
            ni = map_dat(n)% map_init (l) 
            nf = map_dat(n)% map_final(l) 

            lambda = map_dat(n)% lambda(l)

            awarshel_raw(n)% nrg_map(m) = ( 1.0d0 - lambda ) * map_dat_raw(n)% &
                evb_mat(ni,ni,m) + lambda * map_dat_raw(n)% evb_mat(nf,nf,m)

            awarshel_raw(n)% dVmapdlam(m) = map_dat_raw(n)% evb_mat(nf,nf,m) &
                                          - map_dat_raw(n)% evb_mat(ni,ni,m)

         enddo

!  +---------------------------------------------------------------+
!  |  Energy gap reaction coordinate                               |
!  |                                                               |
!  |  RC_egap = V_ni - V_nf                                        |
!  +---------------------------------------------------------------+

         do l = 1, nbias

            ni = map_dat(n)% map_init (l)
            nf = map_dat(n)% map_final(l)

            awarshel_raw(n)% nrg_gap(m) =  map_dat_raw(n)% evb_mat(ni,ni,m) &
                                        -  map_dat_raw(n)% evb_mat(nf,nf,m)

         enddo

      enddo

   enddo 

!  +---------------------------------------------------------------+
!  +---------------------------------------------------------------+

   end subroutine gen_map_pot


! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Allocate storage space for gen_map_pot                         |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine gen_map_pot_alloc

   use pmf_parm, only: nwin, ndata_max

   use map_pot,  only: awarshel_raw

   implicit none

   !  +---------------------------------------------------------------+

   integer :: n, alloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory: Warshel mapping potential biased sampling   |
!  +---------------------------------------------------------------+

   allocate( awarshel_raw(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'awarshel_raw' )

   do n = 1, nwin

      allocate( awarshel_raw(n)% nrg_evb(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_evb in awarshel_raw' )

      allocate( awarshel_raw(n)% nrg_map(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_map in awarshel_raw' )

      allocate( awarshel_raw(n)% nrg_gap(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_gap in awarshel_raw' )

      allocate( awarshel_raw(n)% dVmapdlam(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'dVmapdlam in awarshel_raw' )

   enddo

   end subroutine gen_map_pot_alloc

