! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Generate umbrella potential from mapping potential data        |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine gen_umb_pot 

   use pmf_parm, only: nevb, nbias, nwin, ndata, ndata_max &
                     , dyn_type, nbead, RC_data

   use map_pot,  only: map_dat_raw, map_dat 

   use wham_umb, only: rcrd_win_raw, vel0_raw, RC_raw 


   implicit none

   !  +---------------------------------------------------------------+

   integer :: l, m, n, mm, nn, ni, nf, info 

   real*8 :: evb_vec(nevb,nevb), nrg_evb(nevb) 

   real*8 :: work(26*nevb) 


!  x---------------------------------------------------------------x
!  |  Allocate memory                                              |
!  x---------------------------------------------------------------x

   call gen_umb_pot_alloc

!  x---------------------------------------------------------------x
!  |  V_umb = V_el0 + 0.50 * k_umb * ( nrg_gap - r0_umb )^2        |
!  x---------------------------------------------------------------x

   do n = 1, nwin

      do m = 1, ndata(n)

!  +---------------------------------------------------------------+
!  |  Diagonalize EVB matrix using LAPACK DSYEV                    |
!  +---------------------------------------------------------------+

         evb_vec(:,:) = map_dat_raw(n)% evb_mat(:,:,m)

         call dsyev ( 'V', 'L', nevb, evb_vec, nevb, nrg_evb &
                    , work, 26*nevb, info )

         if( info /= 0 ) then
            write(6,'(A)') 'Problem diagonalizing the EVB ' &
                         //'matrix using LAPACK DSYEV'
            write(6,'(A, I8)') 'info = ', info
            stop 6
         endif

         vel0_raw(m,n) = nrg_evb(1)

      enddo

   enddo

!  +---------------------------------------------------------------+
!  |  Energy gap reaction coordinate: RC_egap = V_ni - V_nf        |
!  +---------------------------------------------------------------+

   select case( trim( adjustl( dyn_type ) ) )

      case( "evb_map", "egap_umb", "pimd_evb_map", "pimd_egap_umb"  )

         do n = 1, nwin
            do m = 1, ndata(n)
               do l = 1, nbias
                  ni = map_dat(n)% map_init (l)
                  nf = map_dat(n)% map_final(l)
                  rcrd_win_raw(m,n) =  map_dat_raw(n)% evb_mat(ni,ni,m) &
                                    -  map_dat_raw(n)% evb_mat(nf,nf,m)
               enddo
            enddo
         enddo

      case( "dbonds_umb", "bond_umb", "pimd_dbonds_umb", "pimd_bond_umb" )

         do n = 1, nwin
            mm = 0
            do m = 1, RC_data(n)
                  do nn = 1, nbead
                  mm = mm + 1 
                  rcrd_win_raw(mm,n) = RC_raw(nbead+1,m,n) / dble( nbead )
                  enddo
            enddo
         enddo

   end select

!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x
!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x

   end subroutine gen_umb_pot


! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Allocate storage space for gen_umb_pot                         |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine gen_umb_pot_alloc

   use pmf_parm, only: nwin, dyn_type, ndata_max, RC_data_max

   use wham_umb, only: rcrd_win_raw, vel0_raw 

   implicit none

   !  +---------------------------------------------------------------+

   integer :: alloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory: mapping potential reformulated as umbrella  |
!  |  biased sampling                                              |
!  +---------------------------------------------------------------+

   allocate( vel0_raw(ndata_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'vel0_raw' )


   allocate( rcrd_win_raw(ndata_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'rcrd_win_raw' )

   end subroutine gen_umb_pot_alloc


! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Deallocate storage space for gen_umb_pot                       |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine gen_umb_pot_dealloc

   use wham_umb, only: rcrd_win_raw, vel0_raw

   implicit none

   !  +---------------------------------------------------------------+

   integer :: dealloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory: mapping potential reformulated as umbrella  |
!  |  biased sampling                                              |
!  +---------------------------------------------------------------+

   deallocate( vel0_raw, rcrd_win_raw, stat = dealloc_error )

   if( dealloc_error /= 0 ) then
      print *, 'Problem deallocating vel0_raw, rcrd_win_raw'
      stop 6
   endif 

   end subroutine gen_umb_pot_dealloc

