! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Compute Warshel quantities                                     |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#


   subroutine get_warshel ( POB, RCRD, VEL11, VEL22, VEL0, VMAP
     &                        , EBW, EBFN, NDATA, NMAX, MLAMBDA
     &                        , BINSIZE, NUMBIN, BINSTART, CPARAM
     &                        , BETA, EXP_TOL, IOGR )


   INTEGER MLAMBDA, NUMBIN, NDATA(MLAMBDA)

   REAL*8 EBW(MLAMBDA,NMAX,MLAMBDA), EBFN(MLAMBDA) &
        , RCRD(NMAX,MLAMBDA), EBF(MLAMBDA)

   REAL*8  VEL0(NMAX,MLAMBDA),  VMAP(NMAX,MLAMBDA) &
        , VEL11(NMAX,MLAMBDA), VEL22(NMAX,MLAMBDA)

   REAL*8  POBF(NUMBIN,MLAMBDA),      POB(NUMBIN,MLAMBDA) &
        ,  DTE0(NUMBIN,MLAMBDA),  DTE0PMF(NUMBIN,MLAMBDA) &
        , DTE11(NUMBIN,MLAMBDA), DTE11PMF(NUMBIN,MLAMBDA) &
        , DTE22(NUMBIN,MLAMBDA), DTE22PMF(NUMBIN,MLAMBDA)

   REAL*8 PMF_WARSHEL(NUMBIN,MLAMBDA) &
        , G11_WARSHEL(NUMBIN,MLAMBDA) &
        , G22_WARSHEL(NUMBIN,MLAMBDA)

   REAL*8 RCRD_BIN(NUMBIN), PMF_WPMF(NUMBIN) &
        , HITS_ETA(NUMBIN,MLAMBDA)
    
   REAL*8 CPARAM(MLAMBDA,2), BOTTOM, BINSIZE, BINSTART, LAM, BETA

   CHARACTER FNAME*12


   BETA_INV  = 1.0D0 / BETA
   SFACTOR   = 1.0D0 / 4.184D0
   HITS_TOL  = 100

!  +---------------------------------------------------------------+
!  |  Populate the RCRD array                                      |
!  +---------------------------------------------------------------+

   DO N = 1, NUMBIN

      RCRD_BIN(N) = ( BINSTART + BINSIZE * ( DBLE(N) + 0.5D0 ) ) &
                  * SFACTOR

   ENDDO

!  +---------------------------------------------------------------+
!  |  Compute the optimal biased distribution function             |
!  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!  |  p_k(eta) = exp( - beta * ( W_k(eta) - f_k ) ) * p_0(eta)     |
!  +---------------------------------------------------------------+

   CALL WARSHEL_POBF ( POBF, POB, RCRD, EBW, EBFN, NDATA, NMAX &
                     , MLAMBDA, BINSIZE, NUMBIN, BINSTART )

!  +---------------------------------------------------------------+
!  |  Compute average energy differences (V_el0 - V_map),          |
!  |     (V_11 - V_map), (V_22 - V_map)                            |
!  +---------------------------------------------------------------+

   CALL WARSHEL_EDIFF ( DTE0, HITS_ETA, RCRD, VEL0, VMAP &
                      , NDATA, NMAX, MLAMBDA, BINSIZE, NUMBIN &
                      , BINSTART, BETA, EXP_TOL ) 

   CALL WARSHEL_EDIFF ( DTE11, HITS_ETA, RCRD, VEL11, VMAP &
                      , NDATA, NMAX, MLAMBDA, BINSIZE, NUMBIN &
                      , BINSTART, BETA, EXP_TOL )

   CALL WARSHEL_EDIFF ( DTE22, HITS_ETA, RCRD, VEL22, VMAP &
                      , NDATA, NMAX, MLAMBDA, BINSIZE, NUMBIN &
                      , BINSTART, BETA, EXP_TOL )

!  +---------------------------------------------------------------+
!  |  Output DTE0 distribution and corresponding PMF               |
!  +---------------------------------------------------------------+

   FNAME = '       DTE0_'
   CALL OUT_PMFLAM ( DTE0, RCRD_BIN, CPARAM, HITS_ETA &
                   , HITS_TOL, MLAMBDA, NUMBIN, FNAME, .TRUE. )

   DO J = 1, MLAMBDA

      DO N = 1, NUMBIN

         IF( DTE0(N,J) .GT. 0.0D0 ) THEN 

           DTE0PMF(N,J) =  - DLOG( DTE0(N,J) ) * BETA_INV * SFACTOR

         ENDIF 

      ENDDO

   ENDDO

   FNAME = '    DTE0PMF_'

   CALL OUT_PMFLAM ( DTE0PMF, RCRD_BIN, CPARAM, HITS_ETA &
                   , HITS_TOL, MLAMBDA, NUMBIN, FNAME, .FALSE. )

!  +---------------------------------------------------------------+
!  |  Output PMF_WARSHEL, G11_WARSHEL & G22_WARSHEL                |
!  +---------------------------------------------------------------+

   CALL WARSHEL_PMF ( PMF_WARSHEL, POBF, DTE0, MLAMBDA &
                    , NUMBIN, BETA, SFACTOR )

   FNAME = 'PMF_WARSHEL_'
   CALL OUT_PMFLAM ( PMF_WARSHEL, RCRD_BIN, CPARAM, HITS_ETA &
                   , HITS_TOL, MLAMBDA, NUMBIN, FNAME, .FALSE. )

   CALL WARSHEL_PMF ( G11_WARSHEL, POBF, DTE11, MLAMBDA &
                    , NUMBIN, BETA, SFACTOR )

   FNAME = 'G11_WARSHEL_'
   CALL OUT_PMFLAM ( G11_WARSHEL, RCRD_BIN, CPARAM, HITS_ETA &
                   , HITS_TOL, MLAMBDA, NUMBIN, FNAME, .FALSE. )

   CALL WARSHEL_PMF ( G22_WARSHEL, POBF, DTE22, MLAMBDA &
                    , NUMBIN, BETA, SFACTOR )

   FNAME = 'G22_WARSHEL_'
   CALL OUT_PMFLAM ( G22_WARSHEL, RCRD_BIN, CPARAM, HITS_ETA &
                   , HITS_TOL, MLAMBDA, NUMBIN, FNAME, .FALSE. )

!  +---------------------------------------------------------------+
!  |  Recombine unbiased distributions using W-PMF                 |
!  +---------------------------------------------------------------+

   CALL WPMF ( PMF_WPMF, PMF_WARSHEL, HITS_ETA, MLAMBDA &
             , NUMBIN, HITS_TOL )

   FNAME = 'PMF_WPMF.ETA'
   CALL OUT_PMFETA ( PMF_WPMF, RCRD_BIN, FNAME, NUMBIN )


!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x
!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x


   end subroutine get_warshel

