! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Read in EVB data file                                          |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine io_evb 

   use pmf_parm, only: nevb, nbias, nwin, nbead, io_unit &
                     , dyn_type, evb_step, dt, f_k_thermint

   use map_pot,  only: map_dat_raw, map_dat

   use wham_umb, only: k_umb, r0_umb, dbonds_RC, bond_RC, RC_raw

   use pimd_dat, only: pimd_etot, pimd_ekin, pimd_epot &
                     , vel0_etot, vel0_ekin, vel0_epot

   implicit none


   !  +---------------------------------------------------------------+

   integer :: k, l, m, n, ll, mm, nn, ios, mevb, mbias 

   real*8 :: evb_tmp(nevb*nevb)

   logical :: done_header 

   character(13)  :: cdum

   character(512) :: cline


!  +---------------------------------------------------------------+
!  |  Allocate memory                                              |
!  +---------------------------------------------------------------+

   call io_evb_alloc

!  +---------------------------------------------------------------+
!  |  Open EVB data file                                           |
!  +---------------------------------------------------------------+

   do n = 1, nwin 

      open( io_unit, file = trim( adjustl(  map_dat(n)% evb_file  ) ) )

!  +---------------------------------------------------------------+
!  |  Read mapping potential data                                  |
!  +---------------------------------------------------------------+

      done_header = .false. 

      do while( .not. done_header )

         read(io_unit,'(A)') cline

         select case( trim( adjustl(cline) ) )

            case( "[DYNAMICS TYPE]:" )
               read(io_unit,'(A)') dyn_type

            case( "[NBEAD]:" )
               read(io_unit,'(I8)') nbead
               if( nbead == 0 ) nbead = 1

           case( "[NEV]   [NBIAS]   [NTW_EVB]   [DT]:" )
               read(io_unit, 1000) mevb, mbias, evb_step, dt
               read(io_unit,'(/)')
               read(io_unit,'(A)')
               done_header = .true. 

           case( "[NEV]   [NBIAS]   [EVB_STEP]   [DT]:" )
               read(io_unit, 1000) mevb, mbias, evb_step, dt
               read(io_unit,'(/)')
               read(io_unit,'(A)')
               done_header = .true.

         end select

      enddo 

      select case( trim( adjustl(dyn_type) ) )

         case( "evb_map", "pimd_evb_map" )

            do m = 1, nbias
               read(io_unit,2000,iostat=ios) map_dat(n)% map_init(m) &
                   , map_dat(n)% map_final(m), map_dat(n)% lambda(m)
            enddo
            call chk_ios( ios, 'MAP_POT', map_dat(n)% evb_file )

         case( "egap_umb", "pimd_egap_umb" )

            f_k_thermint = .false. 

            do m = 1, nbias
               read(io_unit,2500,iostat=ios) map_dat(n)% map_init(m) &
                   , map_dat(n)% map_final(m), k_umb(n), r0_umb(n)
            enddo
            call chk_ios( ios, 'EGAP_UMB', map_dat(n)% evb_file )

         case( "dbonds_umb", "pimd_dbonds_umb" )

            f_k_thermint = .false.

            do m = 1, nbias
               read(io_unit,5000,iostat=ios) dbonds_RC(m,n)% iatom &
                           , dbonds_RC(m,n)% jatom &
                           , dbonds_RC(m,n)% katom, k_umb(n), r0_umb(n)
            enddo
            call chk_ios( ios, 'DBONDS_UMB', map_dat(n)% evb_file )

         case( "bond_umb", "pimd_bond_umb" )

            f_k_thermint = .false.

            do m = 1, nbias
               read(io_unit,5000,iostat=ios) bond_RC(m,n)% iatom &
                           , bond_RC(m,n)% jatom, k_umb(n), r0_umb(n)
            enddo
            call chk_ios( ios, 'BOND_UMB', map_dat(n)% evb_file )

      end select


      m   = 0
      ll  = 0
      mm  = 0
      nn  = 0
      ios = 0
      do while( ios == 0 ) 

         read(io_unit,'(A)', iostat=ios) cline

         if( trim( adjustl(cline) ) == '[EVB MATRIX]' ) then

            read(io_unit,3000,iostat=ios) ( evb_tmp(l), l = 1, nevb * nevb )

            m = m + 1

            map_dat_raw(n)% evb_mat(:,:,m) = reshape( evb_tmp(:), (/ nevb, nevb /) )
            call chk_ios( ios, 'EVB_MAT', map_dat(n)% evb_file ) 

         endif 

         if( trim( adjustl(cline) ) == '[RC_BEAD]' ) then

            do l = 1, nbias
               ll = ll + 1
               read(io_unit,3000,iostat=ios) ( RC_raw(k,ll,n), k = 1, nbead + 1 ) 
            enddo

         endif

         if( cline(1:12) == '{EMAP_PIMD}:' .or. cline(1:12) == '{EGAP_PIMD}:') then

            mm = mm + 1
            backspace( io_unit ) 
            read(io_unit,4000,iostat=ios) cdum, pimd_etot(mm,n), pimd_ekin(mm,n), pimd_epot(mm,n)

         endif 

         if( cline(1:12) == '{VEL0_PIMD}:' ) then

            nn = nn + 1
            backspace( io_unit )
            read(io_unit,4000,iostat=ios) cdum, vel0_etot(nn,n), vel0_ekin(nn,n), vel0_epot(nn,n)

         endif

      enddo

      close( io_unit )

   enddo


 1000 format( 3(2X,I8), 2X, F14.8 )
 2000 format( 2I8, F14.8 )
 2500 format( 2I8, 2F14.8 )
 3000 format( (5(2X,F14.8)) )
 4000 format( A, 3(2X, F14.8) )
 5000 format( 3I8, F14.8, 2X, F14.8 )
 6000 format( 2I8, F14.8, 2X, F14.8 )

!  +---------------------------------------------------------------+
!  +---------------------------------------------------------------+

   end subroutine io_evb


! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Allocate storage space for io_evb                              |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine io_evb_alloc

   use pmf_parm, only: nevb, nbias, nwin, nbead, ndata_max, RC_data_max

   use map_pot,  only: map_dat_raw, map_dat

   use wham_umb, only: k_umb, r0_umb, dbonds_RC, bond_RC, RC_raw

   use pimd_dat, only: pimd_etot, pimd_ekin, pimd_epot &
                     , vel0_etot, vel0_ekin, vel0_epot

   implicit none

   !  +---------------------------------------------------------------+

   integer :: n, alloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory: EVB mapping potential data                  |
!  +---------------------------------------------------------------+

   allocate( map_dat_raw(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'map_dat_raw' )

   do n = 1, nwin

      allocate( map_dat_raw(n)% evb_mat(nevb,nevb,ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'evb_mat in map_dat_raw' )

   enddo

   do n = 1, nwin

      allocate( map_dat(n)% evb_mat(nevb,nevb,ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'evb_mat in map_dat' )

      allocate( map_dat(n)% lambda(nbias), stat = alloc_error )
      call chk_alloc ( alloc_error, 'lambda in map_dat' )

      allocate( map_dat(n)% map_init(nbias), stat = alloc_error )
      call chk_alloc ( alloc_error, 'map_init in map_dat' )

      allocate( map_dat(n)% map_final(nbias), stat = alloc_error )
      call chk_alloc ( alloc_error, 'map_final in map_dat' )

   enddo

!  +---------------------------------------------------------------+
!  |  Allocate memory: dbonds_RC and bond_RC                       |
!  +---------------------------------------------------------------+

   allocate( dbonds_RC(nbias,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'dbonds_RC' )

   allocate( bond_RC(nbias,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'bond_RC' )

!  +---------------------------------------------------------------+
!  |  Allocate memory: mapping potential reformulated as umbrella  |
!  |  biased sampling                                              |
!  +---------------------------------------------------------------+

   allocate( k_umb(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'k_umb' )

   allocate( r0_umb(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'r0_umb' )

!  +---------------------------------------------------------------+
!  |  Allocate memory: EVB/PIMD                                    |
!  +---------------------------------------------------------------+

   allocate( RC_raw(nbead,RC_data_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'RC_raw' )

   allocate( pimd_etot(ndata_max/nbead,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'pimd_etot' )

   allocate( pimd_ekin(ndata_max/nbead,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'pimd_ekin' )

   allocate( pimd_epot(ndata_max/nbead,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'pimd_epot' )

   allocate( vel0_etot(ndata_max/nbead,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'vel0_etot' )

   allocate( vel0_ekin(ndata_max/nbead,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'vel0_ekin' )

   allocate( vel0_epot(ndata_max/nbead,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'vel0_epot' )


   end subroutine io_evb_alloc


