! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Output EVB data file for comparison with the IO file           |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine out_evbdata 

   use pmf_parm, only: nevb, nbias, nwin, nbead, ndata, io_unit &
                     , dyn_type, evb_step, dt

   use map_pot,  only: map_dat_raw, map_dat

   use wham_umb, only: k_umb, r0_umb

   use pimd_dat, only: pimd_etot, pimd_ekin, pimd_epot &
                     , vel0_etot, vel0_ekin, vel0_epot

   implicit none


   !  +---------------------------------------------------------------+

   integer :: m, n, mm, nn

   character(25) :: cwin


!  +---------------------------------------------------------------+
!  |  Open EVB data file                                           |
!  +---------------------------------------------------------------+

   do n = 1, nwin 

      write( cwin, '(I8)') n
   
      open( io_unit, file = 'EVB_OUTDATA_'//trim( adjustl(cwin) ) )

!  +---------------------------------------------------------------+
!  |  Write mapping potential data                                 |
!  +---------------------------------------------------------------+

      write(io_unit,'(/)')
      write(io_unit, 1000) '  [DYNAMICS TYPE]: ', trim( adjustl(dyn_type) )
      write(io_unit,'(/)')
      write(io_unit, 1500) '  [NBEAD]: ', nbead
      write(io_unit,'(/)')
      write(io_unit, 2000) '  [NEV]   [NBIAS]   [EVB_STEP]   [DT]: ' &
                          , nevb, nbias, evb_step, dt

      select case( trim( adjustl(dyn_type) ) )

         case( "evb_map", "pimd_evb_map" )

            write(io_unit,'(/)')
            do m = 1, nbias
               write(io_unit,3000) '  [MAPPING POTENTIAL]:   ni, nf, lambda ' &
                                   , map_dat(n)% map_init(m), map_dat(n)% map_final(m) &
                                   , map_dat(n)% lambda(m)
               print *, 'Window :: ', n, map_dat(n)% lambda(m)
            enddo

         case( "egap_umb", "pimd_egap_umb" )

            write(io_unit,'(/)')
            do m = 1, nbias
               write(io_unit,3500) '  [NRG_GAP UMBRELLA]:   ni, nf, k, ezero ' &
                                   , map_dat(n)% map_init(m), map_dat(n)% map_final(m) &
                                   , k_umb(n), r0_umb(n)
               print *, 'Window :: ', n, k_umb(n), r0_umb(n)
            enddo

      end select 

      m  = 0
      mm = 0

      do while( mm < ndata(n) ) 

         m = m + 1
         write(io_unit,'(/)')
         write(io_unit, 4000) '{NSTEP}: ', (m-1)*evb_step, '{TIME}: ', (m-1)*evb_step*dt

         do nn = 1, nbead 
            mm = mm + 1
            write(io_unit,'(A)')
            write(io_unit, 5000) '  [EVB MATRIX]', map_dat_raw(n)% evb_mat(:,:,mm)
         enddo 

         select case( trim( adjustl(dyn_type) ) )

            case( "pimd_evb_map" )
               write(io_unit,'(A)')
               write(io_unit,6000) '{EMAP_PIMD}: ', pimd_etot(m,n), pimd_ekin(m,n), pimd_epot(m,n)
               write(io_unit,'(A)')
               write(io_unit,6000) '{VEL0_PIMD}: ', vel0_etot(m,n), vel0_ekin(m,n), vel0_epot(m,n)

            case( "pimd_egap_umb" )
               write(io_unit,'(A)')
               write(io_unit,6000) '{EGAP_PIMD}: ', pimd_etot(m,n), pimd_ekin(m,n), pimd_epot(m,n)
               write(io_unit,'(A)')
               write(io_unit,6000) '{VEL0_PIMD}: ', vel0_etot(m,n), vel0_ekin(m,n), vel0_epot(m,n)

         end select

      enddo

      close( io_unit )

   enddo

 1000 format( A/, A )
 1500 format( A/, I8 )
 2000 format( A/, 3(2X,I8), 2X, F14.8 )
 3000 format( A/, 2I8, F14.8 )
 3500 format( A/, 2I8, F14.8, 2X, F14.8 )
 4000 format( A, 2X, I10, 2X, A, 2X, F14.8 )
 5000 format( A/, (5(2X,F14.8)) )
 6000 format( A, 3(2X, F14.8) )

!  +---------------------------------------------------------------+
!  +---------------------------------------------------------------+

   end subroutine out_evbdata


