! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Read inputs for PMF                                            |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine pmf_input

   use pmf_parm, only: nevb, nbias, nwin, nbead, ndata_max, ndata &
                     , RC_data_max, RC_data &
                     , neject_1stdata, neject_highumb, neject_lowumb &
                     , io_unit, wham_ifile, file_size, nwin_size &
                     , debug, bin_width, bin_width_inv, hits_tol &
                     , temperature, beta, beta_inv, mgc, joule2cal &
                     , f_k_thermint, dyn_type, data_tol &
                     , Lchk_data_slices

   use map_pot,  only: map_dat 

   use wham_umb, only: f_k, wham_tol 

   
   implicit none


   !  x---------------------------------------------------------------x

   integer :: n, mm, nn, ios, alloc_error

   integer :: nhits_tol, ndata_min, RC_data_min

   integer, intrinsic :: maxval, minval

   logical :: done_header

   real*8 :: FE_C(nwin_size) 

   character(512) :: evb_ifile(file_size), cline 

   namelist / evb / nevb, nbias, bin_width, debug, temperature &
                  , nhits_tol, evb_ifile, neject_1stdata &
                  , neject_highumb, neject_lowumb, FE_C, wham_tol &
                  , f_k_thermint, data_tol, Lchk_data_slices

!  +---------------------------------------------------------------+
!  |  Initialize EVB input arrays                                  |
!  +---------------------------------------------------------------+

   nevb  = 2
   nbias = 1

   nhits_tol = 25

   evb_ifile(:) = 'XXX'
   FE_C(:) = 0.0d0 

   neject_1stdata = 0
   neject_highumb = 0
   neject_lowumb  = 0

   debug = .false. 
   f_k_thermint = .false. 
   Lchk_data_slices = .true.

   bin_width = 0.40d0 
   temperature = 300.0d0 
   wham_tol = 1.0d-5
   data_tol = 1.0d-6

!  +---------------------------------------------------------------+
!  |  Read EVB input file                                          |
!  +---------------------------------------------------------------+

   open( io_unit, file = wham_ifile )

   read( io_unit, nml = evb, IOSTAT = ios )

   call chk_ios( ios, 'NAMELIST', wham_ifile )

   close( io_unit ) 

   hits_tol = dble( nhits_tol )
   bin_width_inv = 1.0d0 / bin_width

!  +---------------------------------------------------------------+
!  |  beta = 1 / k_B T                                             |
!  +---------------------------------------------------------------+

   beta = 1.0d0 / ( mgc * temperature * joule2cal )
   beta_inv = 1.0d0 / beta

!  +---------------------------------------------------------------+
!  |  Scan through the windows and count the number of             |
!  |  [EVB MATRIX] data entries in each window                     |
!  +---------------------------------------------------------------+

   nwin = 0

   do n = 1, file_size
      if( trim( adjustl( evb_ifile(n) ) ) .ne. 'XXX' ) nwin = nwin + 1
   enddo

   allocate( f_k(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'f_k' )

   do n = 1, nwin

      f_k(n) = FE_C(n)

   enddo 

   allocate( ndata(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'ndata' )

   allocate( RC_data(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'RC_data' )

   ndata(:) = 0
   RC_data(:) = 0

   do n = 1, nwin 

      open( io_unit, file = trim( adjustl( evb_ifile(n) ) ) )

      done_header = .false.

      do while( .not. done_header )

         read(io_unit,'(A)') cline

         select case( trim( adjustl(cline) ) )

            case( "[DYNAMICS TYPE]:" )
               read(io_unit,'(A)') dyn_type

            case( "[NBEAD]:" )
               read(io_unit,'(I8)') nbead

           case( "[NEV]   [NBIAS]   [NTW_EVB]   [DT]:" )
               done_header = .true.

           case( "[NEV]   [NBIAS]   [EVB_STEP]   [DT]:" )
               done_header = .true.

         end select

      enddo

      if( nbead == 0 ) nbead = 1

      mm  = 0
      nn  = 0
      ios = 0

      do while( ios == 0 ) 

         read( io_unit,'(A)', iostat=ios) cline

         if( trim( adjustl( cline ) ) == '[EVB MATRIX]' ) nn = nn + 1

         if( trim( adjustl( cline ) ) == '[RC_BEAD]' ) mm = mm + 1

      enddo

      ndata(n) = nn 
      RC_data(n) = mm

      close( io_unit )

   enddo 

   ndata_max = maxval( ndata(:) ) 
   ndata_min = minval( ndata(:) ) 

   RC_data_max = maxval( RC_data(:) )
   RC_data_min = minval( RC_data(:) )
   
   print *, 'ndata_max = ', ndata_max
   print *, 'ndata_min = ', ndata_min

   print *, 'RC_data_max = ', RC_data_max
   print *, 'RC_data_min = ', RC_data_min

!  +---------------------------------------------------------------+
!  |  Allocate memory: EVB mapping potential data                  |
!  +---------------------------------------------------------------+

   allocate( map_dat(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'map_dat' )

   do n = 1, nwin 

      map_dat(n)% evb_file = evb_ifile(n)

   enddo 


!  +---------------------------------------------------------------+
!  +---------------------------------------------------------------+

   end subroutine pmf_input




