! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Prune mapping potential data                                   |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine prune_map_pot

   use pmf_parm, only: nevb, nwin, nbead, ndata_max, ndata, num_bin &
                     , bin_width, neject_1stdata, neject_highumb &
                     , neject_lowumb, dyn_type, debug, bin_start, rcrd

   use map_pot,  only: awarshel_type, awarshel, awarshel_raw &
                     , map_dat_type, map_dat, map_dat_raw &
                     , wpmf, wpmf_print

   use pimd_dat, only: pimd_ekin, vel0_ekin 

   implicit none


   !  +---------------------------------------------------------------+

   type( awarshel_type ), allocatable :: awarshel_tmp(:) 

   type( map_dat_type ), allocatable :: map_dat_tmp(:)

   integer :: m, n, mm, nn, alloc_error, dealloc_error

   integer :: ncfg_umbpot(ndata_max), idum(ndata_max)

   real*8 :: nrg_map_tmp(ndata_max)

   real*8 :: min_nrg_gap, max_nrg_gap, bin_end

   real*8, intrinsic :: minval, maxval, min, max, idint

!  x---------------------------------------------------------------x
!  |  Allocate memory                                              |
!  x---------------------------------------------------------------x

   call prune_map_pot_alloc

!  x---------------------------------------------------------------x
!  |  Allocate temporary arrays                                    |
!  x---------------------------------------------------------------x

   allocate( awarshel_tmp(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'awarshel_tmp' )

   do n = 1, nwin

      allocate( awarshel_tmp(n)% nrg_evb(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_evb in awarshel_tmp' )
      allocate( awarshel_tmp(n)% nrg_map(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_map in awarshel_tmp' )
      allocate( awarshel_tmp(n)% nrg_gap(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_gap in awarshel_tmp' )
      allocate( awarshel_tmp(n)% dVmapdlam(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'dVmapdlam in awarshel_tmp' )

   enddo

   allocate( map_dat_tmp(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'map_dat_tmp' )

   do n = 1, nwin

      allocate( map_dat_tmp(n)% evb_mat(nevb,nevb,ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'evb_mat in map_dat_tmp' )

   enddo

!  x---------------------------------------------------------------x
!  |  Eject the first neject_1stdata points from a trajectory      |
!  x---------------------------------------------------------------x

!  +---------------------------------------------------------------+
!  |  EVB/PIMD: compute the centroid of all quantities             |
!  +---------------------------------------------------------------+

   if( trim( adjustl( dyn_type ) ) == "pimd_evb_map" ) then 

      do n = 1, nwin

         awarshel(n)%    nrg_evb(:) = 0.0d0
         awarshel(n)%    nrg_map(:) = 0.0d0
         awarshel(n)%    nrg_gap(:) = 0.0d0
         awarshel(n)%  dVmapdlam(:) = 0.0d0
         map_dat(n)% evb_mat(:,:,:) = 0.0d0

         mm = 0
         do m = 1, ndata(n), nbead
            mm = mm + 1
            do nn = 0, nbead - 1
               awarshel(n)%    nrg_evb(mm) = awarshel(n)%    nrg_evb(mm) &
                                           + awarshel_raw(n)%   nrg_evb(m+nn)
               awarshel(n)%    nrg_map(mm) = awarshel(n)%    nrg_map(mm) &
                                           + awarshel_raw(n)%   nrg_map(m+nn)
               awarshel(n)%    nrg_gap(mm) = awarshel(n)%    nrg_gap(mm) &
                                           + awarshel_raw(n)%   nrg_gap(m+nn)
               awarshel(n)%  dVmapdlam(mm) = awarshel(n)%  dVmapdlam(mm) &
                                           + awarshel_raw(n)% dVmapdlam(m+nn)
               map_dat(n)% evb_mat(:,:,mm) = map_dat(n)% evb_mat(:,:,mm) &
                                           + map_dat_raw(n)% evb_mat(:,:,m+nn)
            enddo 

!  +---------------------------------------------------------------+
!  |  H_nuc^(quant) = T_nuc^(quant) + Vel0_centroid                |
!  +---------------------------------------------------------------+

            awarshel(n)% nrg_evb(mm) = awarshel(n)% nrg_evb(mm) + vel0_ekin(mm,n)

         enddo
      enddo

      do n = 1, nwin
         mm = 0
         do m = neject_1stdata + 1, ndata(n) / nbead
            mm = mm + 1
            awarshel(n)%   nrg_evb(mm) = awarshel(n)%   nrg_evb(m)
            awarshel(n)%   nrg_map(mm) = awarshel(n)%   nrg_map(m)
            awarshel(n)%   nrg_gap(mm) = awarshel(n)%   nrg_gap(m)
            awarshel(n)% dVmapdlam(mm) = awarshel(n)% dVmapdlam(m)
            map_dat(n)% evb_mat(:,:,mm) = map_dat(n)% evb_mat(:,:,m)
         enddo
         do m = ndata(n) / nbead - neject_1stdata + 1, ndata(n) 
            awarshel(n)%   nrg_evb(m) = 0.0d0
            awarshel(n)%   nrg_map(m) = 0.0d0
            awarshel(n)%   nrg_gap(m) = 0.0d0
            awarshel(n)% dVmapdlam(m) = 0.0d0
            map_dat(n)% evb_mat(:,:,m) = 0.0d0
         enddo 
         ndata(n) = ndata(n) / nbead - neject_1stdata

         PRINT *, 'PIMD NDATA = ', n, ndata(n), neject_1stdata

      enddo

!  +---------------------------------------------------------------+
!  |  Conventional EVB                                             |
!  +---------------------------------------------------------------+

   else if( trim( adjustl( dyn_type ) ) == "evb_map" ) then

      do n = 1, nwin 

         awarshel(n)%    nrg_evb(:) = 0.0d0
         awarshel(n)%    nrg_map(:) = 0.0d0
         awarshel(n)%    nrg_gap(:) = 0.0d0
         awarshel(n)%  dVmapdlam(:) = 0.0d0
         map_dat(n)% evb_mat(:,:,:) = 0.0d0

         mm = 0
         do m = neject_1stdata + 1, ndata(n)

            mm = mm + 1
            awarshel(n)%   nrg_evb(mm) = awarshel_raw(n)%   nrg_evb(m)
            awarshel(n)%   nrg_map(mm) = awarshel_raw(n)%   nrg_map(m)
            awarshel(n)%   nrg_gap(mm) = awarshel_raw(n)%   nrg_gap(m)
            awarshel(n)% dVmapdlam(mm) = awarshel_raw(n)% dVmapdlam(m)

            map_dat(n)% evb_mat(:,:,mm) = map_dat_raw(n)% evb_mat(:,:,m)

         enddo

         ndata(n) = ndata(n) - neject_1stdata

      enddo 

   endif 

!  x---------------------------------------------------------------x
!  |  Eject the neject_highumb highest mapping potential           |
!  x---------------------------------------------------------------x

   do n = 1, nwin

      do m = 1, ndata(n)

         ncfg_umbpot(m) = m
         nrg_map_tmp(m) = awarshel(n)% nrg_map(m)

      enddo 

      call sort2_kfw ( ndata(n), nrg_map_tmp, ncfg_umbpot, idum ) 

      do m = 1, 20 
         mm = ndata(n) - m - 1
         print *, n, awarshel(n)% nrg_map( ncfg_umbpot(mm) )
      enddo

      do m = 1, ndata(n) - neject_highumb

         mm = ncfg_umbpot(m) 

         awarshel_tmp(n)%   nrg_evb(m) = awarshel(n)%   nrg_evb(mm)
         awarshel_tmp(n)%   nrg_map(m) = awarshel(n)%   nrg_map(mm)
         awarshel_tmp(n)%   nrg_gap(m) = awarshel(n)%   nrg_gap(mm)
         awarshel_tmp(n)% dVmapdlam(m) = awarshel(n)% dVmapdlam(mm)

         map_dat_tmp(n)% evb_mat(:,:,m) = map_dat(n)% evb_mat(:,:,mm)

      enddo

      ndata(n) = ndata(n) - neject_highumb

   enddo

   print *, 'neject_highumb = ', neject_highumb

!  x---------------------------------------------------------------x
!  |  Eject the neject_lowumb lowest mapping potential             |
!  x---------------------------------------------------------------x

   do n = 1, nwin

      mm = 0
      do m = neject_lowumb + 1, ndata(n) 

         mm = mm + 1
         awarshel(n)%   nrg_evb(mm) = awarshel_tmp(n)%   nrg_evb(m)
         awarshel(n)%   nrg_map(mm) = awarshel_tmp(n)%   nrg_map(m)
         awarshel(n)%   nrg_gap(mm) = awarshel_tmp(n)%   nrg_gap(m)
         awarshel(n)% dVmapdlam(mm) = awarshel_tmp(n)% dVmapdlam(m)

         map_dat(n)% evb_mat(:,:,mm) = map_dat_tmp(n)% evb_mat(:,:,m)

      enddo

      ndata(n) = ndata(n) - neject_lowumb

   enddo

   print *, 'neject_lowumb = ', neject_lowumb

!  x---------------------------------------------------------------x
!  |  Deallocate temporary arrays                                  |
!  x---------------------------------------------------------------x

   do n = 1, nwin
      deallocate( awarshel_tmp(n)% nrg_evb,   stat = dealloc_error )
      deallocate( awarshel_tmp(n)% nrg_map,   stat = dealloc_error )
      deallocate( awarshel_tmp(n)% nrg_gap,   stat = dealloc_error )
      deallocate( awarshel_tmp(n)% dVmapdlam, stat = dealloc_error )
      deallocate(  map_dat_tmp(n)% evb_mat,   stat = dealloc_error )
   enddo

   deallocate( awarshel_tmp, map_dat_tmp, stat = dealloc_error )

!  x---------------------------------------------------------------x
!  |  Energy gap reaction coordinate                               |
!  |                                                               |
!  |  RC_egap = V_ni - V_nf                                        |
!  x---------------------------------------------------------------x

   bin_start = 0.0d0
   bin_end   = 0.0d0

   do n = 1, nwin

      min_nrg_gap = minval( awarshel(n)% nrg_gap(:) )
      max_nrg_gap = maxval( awarshel(n)% nrg_gap(:) )

      bin_start = min( min_nrg_gap, bin_start )
      bin_end   = max( max_nrg_gap, bin_end   )

   enddo

   num_bin = idint ( ( bin_end - bin_start ) / bin_width )

   if( debug ) then
      write(6,1000) 'BIN_START = ', bin_start
      write(6,1000) 'BIN_END   = ', bin_end
      write(6,2000) 'NUM_BIN   = ', num_bin
   endif

!  x---------------------------------------------------------------x
!  |  Populate reaction coordinate array                           |
!  x---------------------------------------------------------------x

   allocate( rcrd(num_bin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'rcrd' )

   do n = 1, num_bin

      rcrd(n) = bin_start + bin_width * ( dble(n) + 0.50d0 )

   enddo

 1000 format( A, F12.4 )
 2000 format( A, I8    )



!  +---------------------------------------------------------------+
!  +---------------------------------------------------------------+

   end subroutine prune_map_pot


! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Allocate storage space for prune_map_pot                       |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine prune_map_pot_alloc

   use pmf_parm, only: nevb, nwin, ndata_max

   use map_pot,  only: awarshel_type, map_dat_type &
                     , awarshel, awarshel_raw

   implicit none

   !  +---------------------------------------------------------------+

   integer :: n, alloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory: Warshel mapping potential biased sampling   |
!  +---------------------------------------------------------------+

   allocate( awarshel(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'awarshel' )

   do n = 1, nwin

      allocate( awarshel(n)% nrg_evb(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_evb in awarshel' )

      allocate( awarshel(n)% nrg_map(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_map in awarshel' )

      allocate( awarshel(n)% nrg_gap(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'nrg_gap in awarshel' )

      allocate( awarshel(n)% dVmapdlam(ndata_max), stat = alloc_error )
      call chk_alloc ( alloc_error, 'dVmapdlam in awarshel' )

   enddo


   end subroutine prune_map_pot_alloc



