! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Prune umbrella potential data                                  |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine prune_umb_pot

   use pmf_parm, only: nwin, nbead, ndata_max, ndata, num_bin &
                     , bin_width, neject_1stdata, neject_highumb &
                     , neject_lowumb, dyn_type, debug, bin_start, rcrd 

   use wham_umb, only: V_tot, V_tot_raw, vel0, vel0_raw &
                     , rcrd_win, rcrd_win_raw, umb_pot, umb_pot_raw &
                     , k_umb, r0_umb

   use pimd_dat, only: pimd_ekin, vel0_ekin

   implicit none


   !  +---------------------------------------------------------------+

   integer :: m, n, mm, nn, alloc_error, dealloc_error

   integer :: ncfg_umbpot(ndata_max), idum(ndata_max)

   real*8 :: nrg_umb_tmp(ndata_max)

   real*8 :: min_nrg_gap, max_nrg_gap, bin_end

   real*8, intrinsic :: minval, maxval, min, max, idint

   real*8, allocatable :: V_tot_tmp(:,:), rcrd_win_tmp(:,:) &
                        , umb_pot_tmp(:,:), vel0_tmp(:,:) 


!  x---------------------------------------------------------------x
!  |  Allocate memory                                              |
!  x---------------------------------------------------------------x

   call prune_umb_pot_alloc

!  x---------------------------------------------------------------x
!  |  Allocate temporary arrays                                    |
!  x---------------------------------------------------------------x

   allocate( V_tot_tmp(ndata_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'V_tot_tmp' )
   allocate( umb_pot_tmp(ndata_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'umb_pot_tmp' )
   allocate( rcrd_win_tmp(ndata_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'rcrd_win_tmp' )
   allocate( vel0_tmp(ndata_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'vel0_tmp' )

!  x---------------------------------------------------------------x
!  |  Eject the first neject_1stdata points from a trajectory      |
!  x---------------------------------------------------------------x

!  +---------------------------------------------------------------+
!  |  EVB/PIMD: compute the centroid of all quantities             |
!  +---------------------------------------------------------------+

   if( trim( adjustl( dyn_type ) ) == "pimd_egap_umb" .or. &
       trim( adjustl( dyn_type ) ) == "pimd_dbonds_umb"    ) then

      do n = 1, nwin

             vel0(:,n) = 0.0d0
         rcrd_win(:,n) = 0.0d0
            V_tot(:,n) = 0.0d0
          umb_pot(:,n) = 0.0d0

         mm = 0

         do m = 1, ndata(n), nbead
            mm = mm + 1
            do nn = 0, nbead - 1
                   vel0(mm,n) =     vel0(mm,n) +     vel0_raw(m+nn,n) 
               rcrd_win(mm,n) = rcrd_win(mm,n) + rcrd_win_raw(m+nn,n)
            enddo

!  +---------------------------------------------------------------+
!  |  V_umb = 0.50 * k_umb * ( RC_centroid - RC_0 )^2              |
!  +---------------------------------------------------------------+

            umb_pot(mm,n) =  0.50d0 * k_umb(n) &
                          * ( rcrd_win(mm,n) - r0_umb(n) )**2

              V_tot(mm,n) = vel0(mm,n) + umb_pot(mm,n) 

         enddo
      enddo

      do n = 1, nwin
         mm = 0
         do m = neject_1stdata + 1, ndata(n) / nbead
            mm = mm + 1
               V_tot(mm,n) =    V_tot(m,n)
             umb_pot(mm,n) =  umb_pot(m,n)
            rcrd_win(mm,n) = rcrd_win(m,n)
         enddo

         do m = ndata(n) / nbead - neject_1stdata + 1, ndata(n)
               V_tot(m,n) = 0.0d0
             umb_pot(m,n) = 0.0d0
            rcrd_win(m,n) = 0.0d0
         enddo

         ndata(n) = ndata(n) / nbead - neject_1stdata

         PRINT *, 'PIMD NDATA = ', n, ndata(n), neject_1stdata

      enddo

!  +---------------------------------------------------------------+
!  |  Conventional EVB                                             |
!  +---------------------------------------------------------------+

   else if( trim( adjustl( dyn_type ) ) == "egap_umb" ) then

      do n = 1, nwin 

         mm = 0
         do m = neject_1stdata + 1, ndata(n)

            mm = mm + 1

                vel0(mm,n) =         vel0(m,n)
            rcrd_win(mm,n) = rcrd_win_raw(m,n)

!  +---------------------------------------------------------------+
!  |  V_umb = 0.50 * k_umb * ( RC(q) - RC_0 )^2                    |
!  +---------------------------------------------------------------+

             umb_pot(mm,n) = 0.50d0 * k_umb(n) &
                           * ( rcrd_win(mm,n) - r0_umb(n) )**2
               V_tot(mm,n) =  vel0(mm,n) + umb_pot(mm,n) 

         enddo

         ndata(n) = ndata(n) - neject_1stdata

      enddo 

   endif

!  x---------------------------------------------------------------x
!  |  Eject the neject_highumb highest umbrella potential          |
!  x---------------------------------------------------------------x

   do n = 1, nwin

      do m = 1, ndata(n)

         ncfg_umbpot(m) = m
         nrg_umb_tmp(m) = V_tot(m,n)

      enddo 

      call sort2_kfw ( ndata(n), nrg_umb_tmp, ncfg_umbpot, idum ) 

      do m = 1, 20 
         mm = ndata(n) - m - 1
         print *, n, V_tot(ncfg_umbpot(mm),n)
      enddo

      do m = 1, ndata(n) - neject_highumb

         mm = ncfg_umbpot(m) 

            V_tot_tmp(m,n) =    V_tot(mm,n)
          umb_pot_tmp(m,n) =    umb_pot(mm,n)
         rcrd_win_tmp(m,n) = rcrd_win(mm,n)

      enddo

      ndata(n) = ndata(n) - neject_highumb

   enddo

   print *, 'neject_highumb = ', neject_highumb

!  x---------------------------------------------------------------x
!  |  Eject the neject_lowumb lowest umbrella potential            |
!  x---------------------------------------------------------------x

   do n = 1, nwin

      mm = 0
      do m = neject_lowumb + 1, ndata(n) 

         mm = mm + 1
            V_tot(mm,n) =    V_tot_tmp(m,n)
          umb_pot(mm,n) =    umb_pot_tmp(m,n)
         rcrd_win(mm,n) = rcrd_win_tmp(m,n)

      enddo

      ndata(n) = ndata(n) - neject_lowumb

   enddo

   print *, 'neject_lowumb = ', neject_lowumb

!  x---------------------------------------------------------------x
!  |  Deallocate temporary arrays                                  |
!  x---------------------------------------------------------------x

   deallocate( V_tot_tmp, umb_pot_tmp, rcrd_win_tmp, vel0_tmp &
              , stat = dealloc_error )

!  x---------------------------------------------------------------x
!  |  Energy gap reaction coordinate                               |
!  |                                                               |
!  |  RC_egap = V_ni - V_nf                                        |
!  x---------------------------------------------------------------x

   bin_start =  9999.99
   bin_end   = -9999.99

   do n = 1, nwin

      min_nrg_gap = minval( rcrd_win(:,n) )
      max_nrg_gap = maxval( rcrd_win(:,n) )

      bin_start = min( min_nrg_gap, bin_start )
      bin_end   = max( max_nrg_gap, bin_end   )

   enddo

   num_bin = idint ( ( bin_end - bin_start ) / bin_width ) 

   if( debug ) then
      write(6,1000) 'BIN_START = ', bin_start
      write(6,1000) 'BIN_END   = ', bin_end
      write(6,2000) 'NUM_BIN   = ', num_bin
   endif


!  x---------------------------------------------------------------x
!  |  Populate reaction coordinate array                           |
!  x---------------------------------------------------------------x

   allocate( rcrd(num_bin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'rcrd' )

   do n = 1, num_bin

      rcrd(n) = bin_start + bin_width * ( dble(n) + 0.50d0 )

   enddo

 1000 format( A, F12.4 )
 2000 format( A, I8    )

!  +---------------------------------------------------------------+
!  +---------------------------------------------------------------+

   end subroutine prune_umb_pot


! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Allocate storage space for gen_umb_pot                         |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine prune_umb_pot_alloc

   use pmf_parm, only: nwin, ndata_max

   use wham_umb, only: V_tot, vel0, umb_pot, rcrd_win 

   implicit none

   !  +---------------------------------------------------------------+

   integer :: alloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory for umbrella biased sampling                 |
!  +---------------------------------------------------------------+

   allocate( V_tot(ndata_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'V_tot' )

   allocate( umb_pot(ndata_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'umb_pot' )

   allocate( rcrd_win(ndata_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'rcrd_win' )

   allocate( vel0(ndata_max,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'vel0' )


   end subroutine prune_umb_pot_alloc


! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Deallocate storage space for gen_umb_pot                       |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine prune_umb_pot_dealloc

   use wham_umb, only: V_tot, vel0, umb_pot, rcrd_win

   implicit none

   !  +---------------------------------------------------------------+

   integer :: dealloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory for umbrella biased sampling                 |
!  +---------------------------------------------------------------+

   deallocate( V_tot, umb_pot, rcrd_win, vel0, stat = dealloc_error )

   if( dealloc_error /= 0 ) then 
      print *, 'Problem deallocating V_tot, umb_pot, rcrd_win, vel0'
      stop 6
   endif 

   end subroutine prune_umb_pot_dealloc

