! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Compute free energy constants for each mapping potential       |#
! #|  using thermodynamic integration                                |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine therm_int

   use pmf_parm, only: nbias, nwin, ndata, debug, f_k_thermint

   use map_pot,  only: map_dat, awarshel 

   use wham_umb, only: f_k

   implicit none


   !  +---------------------------------------------------------------+

   integer :: m, n

   real*8 :: dVdl_sum, dVdl2_avg, FE_map_lower, avg_lower, lam_lower &
           , lam, avg, dlam, FE_shift

   real*8, intrinsic :: sum, sqrt


!  +---------------------------------------------------------------+
!  |  < d V_map / d lambda >                                       |
!  |  sigma = sqrt( <x^2> - <x>^2 )                                |
!  +---------------------------------------------------------------+

   do n = 1, nwin 

      dVdl_sum  = sum( awarshel(n)% dVmapdlam(:) )

      dVdl2_avg = sum( awarshel(n)% dVmapdlam(:)**2 ) / dble( ndata(n) )

      awarshel(n)% dVdl_avg = dVdl_sum / dble( ndata(n) )
      
      awarshel(n)% dVdl_sd = sqrt( dVdl2_avg - awarshel(n)% dVdl_avg**2 )

   enddo

!  +---------------------------------------------------------------+
!  |  Compute relative free energy constants between each mapping  |
!  |  potential                                                    |
!  +---------------------------------------------------------------+

   FE_map_lower = 0.0d0 
   avg_lower    = 0.0d0
   lam_lower    = 0.0d0

   do n = 1, nwin

      do m = 1, nbias 

         lam = map_dat(n)% lambda(m) 

      enddo

      avg = awarshel(n)% dVdl_avg

      dlam = 0.50d0 * ( lam - lam_lower )
     
      awarshel(n)% FE_map = dlam * ( avg_lower + avg ) + FE_map_lower

      lam_lower = lam
      avg_lower = avg 
      FE_map_lower = awarshel(n)% FE_map

   enddo

   FE_shift = awarshel(1)% FE_map
   awarshel(:)% FE_map = awarshel(:)% FE_map - FE_shift

   if( debug ) then

      do n = 1, nwin 

         write(6,1000) n, awarshel(n)% FE_map, map_dat(n)% lambda(1) &
                     , awarshel(n)% dVdl_avg, awarshel(n)% dVdl_sd

      enddo

   endif 

!  +---------------------------------------------------------------+
!  |  Set estimates for the umbrella f_k based on TI results       |
!  +---------------------------------------------------------------+

   if( f_k_thermint ) then

      do n = 1, nwin 

         f_k(n) = awarshel(n)% FE_map

      enddo 

   endif


 1000 format( I8, 4(2X,F18.4) )

!  +---------------------------------------------------------------+
!  +---------------------------------------------------------------+

   end subroutine therm_int




