! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Combine all the FE pieces using the weighted-PMF method        |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine w_pmf

   use pmf_parm, only: nwin, num_bin, hits_tol

   use map_pot,  only: pmf, wpmf, wpmf_print


   implicit none


   !  x---------------------------------------------------------------x

   integer :: m, n

   real*8  :: weight(num_bin,nwin), bottom


!  x---------------------------------------------------------------x
!  |  Compute weights                                              |
!  x---------------------------------------------------------------x

   call w_pmf_alloc

!  +---------------------------------------------------------------+
!  |  Compute weights                                              |
!  +---------------------------------------------------------------+

   weight(:,:) = 0.0d0 

   do m = 1, num_bin

      bottom = 0.0d0

      do n = 1, nwin

         if( pmf(n)% pb(m) >= hits_tol ) bottom = bottom + pmf(n)% pb(m) 

      enddo

      do n = 1, nwin

         if( pmf(n)% pb(m) >= hits_tol ) weight(m,n) = pmf(n)% pb(m) / bottom

      enddo

   enddo

!  +---------------------------------------------------------------+
!  |  Weight the PMF segments                                      |
!  +---------------------------------------------------------------+

   wpmf(:) = 0.0d0
   wpmf_print(:) = 0.0d0

   do m = 1, num_bin

      do n = 1, nwin 

         if( pmf(n)% pb(m) >= hits_tol ) then

            wpmf(m) = wpmf(m) + pmf(n)% FE_el0(m) * weight(m,n)

            wpmf_print(m) = 1.0d0

         endif   

      enddo 

   enddo 

!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x
!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x

   end subroutine w_pmf


! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Allocate storage space for w_pmf                               |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine w_pmf_alloc

   use pmf_parm, only: num_bin

   use map_pot,  only: wpmf, wpmf_print

   implicit none

   !  +---------------------------------------------------------------+

   integer :: alloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory for PMF calculation                          |
!  +---------------------------------------------------------------+

   allocate( wpmf(num_bin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'wpmf' )

   allocate( wpmf_print(num_bin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'wpmf_print' )

   end subroutine w_pmf_alloc


