! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  F(eta) = integral dR delta[ eta - eta(R) ] F(R)                |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine wham_data 

   use pmf_parm, only: nwin, num_bin, ndata, bin_start, bin_width_inv &
                     , beta

   use wham_umb, only: umb_pot, rcrd_win, pb, ebweta 

   implicit none


   !  +---------------------------------------------------------------+

   integer :: m, n, nb

   real*8, intrinsic :: idint, exp 


!  x---------------------------------------------------------------x
!  |  Allocate memory                                              |
!  x---------------------------------------------------------------x

   call wham_data_alloc

!  +---------------------------------------------------------------+
!  |  Compute exp( - beta * w_n(eta) ) and biased distributions    |
!  +---------------------------------------------------------------+

   ebweta(:,:) = 0.0d0
   pb    (:,:) = 0.0d0

   do n = 1, nwin

      do m = 1, ndata(n)

         nb = idint( ( rcrd_win(m,n) - bin_start ) * bin_width_inv )

         if( nb /= 0 ) then 

            ebweta(nb,n) = ebweta(nb,n) + exp( - beta * umb_pot(m,n) )
            pb    (nb,n) = pb    (nb,n) + 1.0d0

         endif 

      enddo

   enddo

   do n = 1, nwin

      do m = 1, num_bin

         if( pb(m,n) > 0.0d0 ) &
            ebweta(m,n) = ebweta(m,n) / pb(m,n)

      enddo

   enddo

!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x
!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x

   end subroutine wham_data


! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Allocate storage space for wham_data                           |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine wham_data_alloc

   use pmf_parm, only: nwin, num_bin

   use wham_umb, only: pb, ebweta

   implicit none

   !  +---------------------------------------------------------------+

   integer :: alloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory for umbrella biased sampling                 |
!  +---------------------------------------------------------------+

   allocate( pb(num_bin,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'pb' )

   allocate( ebweta(num_bin,nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'ebweta' )

   end subroutine wham_data_alloc



! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Deallocate storage space for wham_data                         |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine wham_data_dealloc

   use wham_umb, only: pb, ebweta

   implicit none

   !  +---------------------------------------------------------------+

   integer :: dealloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory for umbrella biased sampling                 |
!  +---------------------------------------------------------------+

   deallocate( pb, ebweta, stat = dealloc_error )

   if( dealloc_error /= 0 ) then 
      print *, 'Problem deallocating pb and ebweta'
      stop 6
   endif 

   end subroutine wham_data_dealloc


