! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Generate energy gap RC, mapping potential, and EVB             |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine wham_dist 

   use pmf_parm, only: nwin, ndata, bin_start, bin_width_inv, num_bin &
                     , pmf_unit, beta_inv 

   use wham_umb, only: rcrd_win, ebw, ebfn, pb, pub


   implicit none

   !  +---------------------------------------------------------------+

   integer :: i, j, k, l, n, ibin

   real*8  :: ebw_rcrd(num_bin,nwin), bottom, pnorm_inv &
            , f_k, ef_k, fshift

   integer, intrinsic :: idint

   real*8, intrinsic  :: dble, sum 


!  +---------------------------------------------------------------+
!  |  Compute the biased distribution                              |
!  +---------------------------------------------------------------+

   pb(:,:) = 0.0d0

   do j = 1, nwin 

      do n = 1, ndata(j)

         ibin = idint( ( rcrd_win(n,j) - bin_start ) * bin_width_inv )

         pb(ibin,j) = pb(ibin,j) + 1.0d0

      enddo

   enddo

!  +---------------------------------------------------------------+
!  |  Compute the unbiased distribution function                   |
!  +---------------------------------------------------------------+

   pub (:) = 0.0d0 

   do i = 1, nwin 

      do l = 1, ndata(i)

         ibin = idint( ( rcrd_win(l,i) - bin_start ) * bin_width_inv )

         bottom = 0.0d0 

         do j = 1, nwin

            bottom = bottom + ebw(i,l,j) * ebfn(j) 

         enddo 

         if( bottom /= 0.0d0 ) then 

           pub(ibin) = pub(ibin) + 1.0d0 / bottom

               ELSE

                  WRITE(85,*) IBIN, BOTTOM, PUB(IBIN)

             endif 

      enddo 

   enddo 

!  +---------------------------------------------------------------+
!  |  Compute EBW as a function of reaction coordinate             |
!  +---------------------------------------------------------------+

   ebw_rcrd(:,:) = 0.0d0

   do k = 1, nwin

      do i = 1, nwin

         do l = 1, ndata(i)

            ibin = idint( ( rcrd_win(l,i) - bin_start ) * bin_width_inv )

            ebw_rcrd(ibin,k) = ebw_rcrd(ibin,k) + ebw(i,l,k)

         enddo

      enddo

   enddo

!  +---------------------------------------------------------------+
!  |  Check the F_k                                                |
!  +---------------------------------------------------------------+

   PRINT *, 'CHECKING FREE ENERGY CONSTANTS'
   PRINT *, 'K, F_K'

   pnorm_inv = 1.0d0 / sum( pub(:) )

   do k = 1, nwin

      ef_k = 0.0d0

      do n = 1, num_bin

         ef_k = ef_k + ebw_rcrd(n,k) * pub(n) * pnorm_inv

      enddo

      f_k = -log( ef_k ) * beta_inv

      if( k == 1 ) fshift = f_k

      print *, k, f_k - fshift

   enddo


!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x
!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x

   end subroutine wham_dist

