! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Generate PMF by WHAM                                           |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine wham_iter 

   use pmf_parm, only: nwin, ndata, beta, beta_inv

   use wham_umb, only: ebfn, ebw, f_k, wham_tol

   implicit none


   !  +---------------------------------------------------------------+

   integer :: i, j, k, l, m, iter

   real*8 :: ebf(nwin), ebf_old(nwin) &
           , ebfk, delta, bottom 

   real*8, intrinsic :: dble, exp, abs, log 

   logical :: converged


!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x
!  |  Compute free energy constants F_N by iteration               |
!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x

!  +---------------------------------------------------------------+
!  |  0:  Initialize N_n * exp( + beta * F_n )                     |
!  +---------------------------------------------------------------+

   do k = 1, nwin

      print *, 'f_k = ', k, f_k(k)

      ebf (k) = exp( beta * f_k(k) )
      ebfn(k) = dble( ndata(k) ) * ebf(k)

   enddo

   ebf(1) = ebfn(1) / dble( ndata(1) )

   converged = .false.

   iter = 0

   do while ( .not. converged ) 

      iter = iter + 1 

      do k = 1, nwin 

         ebfk = 0.0d0

         do i = 1, nwin

            do l = 1, ndata(i)

               bottom = 0.0d0 

               do j = 1, nwin

                  bottom = bottom + ebw(i,l,j) * ebfn(j) 

!                 write(500,*) bottom, ebw(i,l,j), ebfn(j)

               enddo 

               if( bottom /= 0.0d0 ) ebfk = ebfk + ebw(i,l,k) / bottom

            enddo 

         enddo

         ebf_old(k) = ebf(k)
         ebf    (k) = 1.0d0 / ( ebf(1) * ebfk )
         ebfn   (k) = dble( ndata(k) ) * ebf(k)

      enddo 

!  +---------------------------------------------------------------+
!  |  Check for convergence                                        |
!  +---------------------------------------------------------------+

      converged = .true. 

      print *
      print *, '>>>> ITER <<<<<< ',  ITER

      do k = 1, nwin

         delta = abs( beta_inv * log( ebf(k) / ebf_old(k) ) )

         write(6,111) k, beta_inv * log( ebf    (k) ) &
                       , beta_inv * log( ebf_old(k) ) &
                       , delta

         if( delta >= wham_tol ) converged = .false.

      enddo 

   enddo 


   PRINT *
   PRINT *, 'THE CONVERGED F_K ARE .... '

   do m = 1, nwin

      print *, m, beta_inv * log( ebf(m) )

   enddo

 111  FORMAT( I8, 2X, 2(F20.12, 2X), D15.5 )


!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x
!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x

   end subroutine wham_iter

