! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Generate PMF by WHAM                                           |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine wham_iter_RC 

   use pmf_parm, only: nwin, num_bin, ndata, beta, beta_inv, hits_tol

   use wham_umb, only: pb, pub, ebweta, ebfn, f_k, wham_tol, wham_print

   implicit none


   !  +---------------------------------------------------------------+

   integer :: m, n, iter

   real*8  :: ebf(nwin), ebf_old(nwin), ebfk, delta

   real*8  :: numer(num_bin), denom(num_bin)

   real*8, intrinsic :: dble, exp, abs, log

   logical :: converged


!  x---------------------------------------------------------------x
!  |  Allocate memory                                              |
!  x---------------------------------------------------------------x

   call wham_iter_RC_alloc

!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x
!  |  Compute free energy constants F_N by iteration               |
!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x

!  +---------------------------------------------------------------+
!  |  0:  Initialize N_n * exp( + beta * F_n )                     |
!  +---------------------------------------------------------------+

   do n = 1, nwin

      ebf (n) = exp( beta * f_k(n) )
      ebfn(n) = dble( ndata(n) ) * ebf(n)

   enddo

   converged = .false.

   iter = 0

   do while ( .not. converged ) 

      iter = iter + 1 

!  +---------------------------------------------------------------+
!  |  1:  Compute the unbiased distribution function               |
!  +---------------------------------------------------------------+

      pub  (:) = 0.0d0
      numer(:) = 0.0d0
      denom(:) = 0.0d0

      wham_print(:) = 0.0d0 
 
      do m = 1, num_bin

         do n = 1, nwin

            numer(m) = numer(m) + pb(m,n)
            denom(m) = denom(m) + ebweta(m,n) * ebfn(n)

            if( pb(m,n) >= hits_tol ) wham_print(m) = 1.0d0

         enddo

         if( denom(m) /= 0.0d0) pub(m) = numer(m) / denom(m)

      enddo

!  +---------------------------------------------------------------+
!  |  2:  Compute free energy constants                            |
!  +---------------------------------------------------------------+

      do n = 1, nwin

         ebfk = 0.0d0

         do m = 1, num_bin

            ebfk = ebfk + ebweta(m,n) * pub(m) 

         enddo

         ebf_old(n) = ebf(n)
         ebf    (n) = 1.0d0 / ( ebf(1) * ebfk )
         ebfn   (n) = dble( ndata(n) ) * ebf(n)

      enddo 

!  +---------------------------------------------------------------+
!  |  Check for convergence                                        |
!  +---------------------------------------------------------------+

      converged = .true. 

      print *
      print *, '>>>> ITER <<<<<< ',  ITER

      do n = 1, nwin

         delta = abs( beta_inv * log( ebf(n) / ebf_old(n) ) )

         write( 6, 111 ) n, beta_inv * log( ebf    (n) ) &
                          , beta_inv * log( ebf_old(n) ) &
                          , delta

         if( delta >= wham_tol ) converged = .false.

      enddo 

   enddo 

   print *
   print *, 'THE CONVERGED F_K ARE .... '

   do n = 1, nwin

      write(6,888) 'FE_C(', n, ') = ', beta_inv * log( ebf(n) )

   enddo

 111  FORMAT( I8, 2X, 2(F20.12, 2X), D15.5 )
 888  FORMAT( A, I3, A, F10.4 )

!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x
!  x+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++x


   end subroutine wham_iter_RC

! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#
! #|  Allocate storage space for wham_iter_RC                        |#
! #+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#

   subroutine wham_iter_RC_alloc

   use pmf_parm, only: nwin, num_bin

   use wham_umb, only: pub, ebfn, wham_print

   implicit none

   !  +---------------------------------------------------------------+

   integer :: alloc_error

!  +---------------------------------------------------------------+
!  |  Allocate memory for umbrella biased sampling                 |
!  +---------------------------------------------------------------+

   allocate( pub(num_bin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'pub' )

   allocate( ebfn(nwin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'ebfn' )

   allocate( wham_print(num_bin), stat = alloc_error )
   call chk_alloc ( alloc_error, 'wham_print' )

   end subroutine wham_iter_RC_alloc


